package	jp.co.sds_corp.nanchart.graph;

import java.awt.Rectangle;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Vector;

/**
DefaultStorageは、グラフのためのデータ格納クラスです。
*/
public class DefaultStorage extends TreeMap implements GraphStorage
{
	private static final long serialVersionUID = 1L;

	/** X軸の最小値 */
	Object		xMin = null;
	/** X軸の最大値 */
	Object		xMax = null;
	/** Y軸の最小値 */
	Object		yMin = null;
	/** Y軸の最大値 */
	Object		yMax = null;
	/** X軸の値の型 */
	int			nTypeX = -1;
	/** Y軸の値の型 */
	int			nTypeY = -1;

	Vector<Object>		vecSrs = new Vector<Object>();
	TreeMap<Object,Number>		mapKey = new TreeMap<Object,Number>();

	/**
	* デフォルトの状態でDefaultStorageを構築します。
	*/
	public DefaultStorage() {
	}

	/**
	* X軸とY軸の範囲を指定して、DefaultStorageを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public DefaultStorage( Object xMin, Object xMax, Object yMin, Object yMax ) {
		setXRange( xMin, xMax );
		setYRange( yMin, yMax );
	}

	/**
	* X軸とY軸の範囲を指定して、tackStorageを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public DefaultStorage( Object xMin, Object xMax, double yMin, double yMax ) {
		setXRange( xMin, xMax );
		setYRange( Double.valueOf( yMin ), Double.valueOf( yMax ));
	}

	/**
	* X軸とY軸の範囲を指定して、DefaultStorageを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public DefaultStorage( double xMin, double xMax, double yMin, double yMax ) {
		setXRange( Double.valueOf( xMin ), Double.valueOf( xMax ));
		setYRange( Double.valueOf( yMin ), Double.valueOf( yMax ));
	}

	/**
	* X軸の範囲を設定します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	*/
	public void setXRange( Object xMin, Object xMax ) {
		this.xMax = xMax;
		this.xMin = xMin;
	}

	/**
	* Y軸の範囲を設定します。
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public void setYRange( Object yMin, Object yMax ) {
		this.yMax = yMax;
		this.yMin = yMin;
	}

	/**
	* 保持しているデータから、X軸の範囲を設定します。
	*/
	public void setXRange() {
		setXRange( mapKey.firstKey(), mapKey.lastKey() );
	}

	/**
	* 保持しているデータから、Y軸の範囲を設定します。
	*/
	public void setYRange() {
		Value		dMax = new Value( Value.getMinValue( nTypeY ));
		Iterator<Object>	itrSrs;
		Object		srs;
		Iterator<Object>	itr;
		Map<Object,Number>			mapKey;
		Map			mapVal;

		itrSrs = getIterator();
		while ( itrSrs.hasNext() ) {
			srs = itrSrs.next();
			mapVal = getMapVal( srs );
			mapKey = getKeys();
			itr = mapKey.keySet().iterator();
			while ( itr.hasNext() ) {
				Object key = itr.next();
				Value cur = (Value )mapVal.get( key );
				if ( cur == null ) {
					continue;
				}
				if ( dMax.compareTo( cur ) < 0 ) {
					dMax = new Value( cur );
				}
			}
		}
		Value		dMin = new Value( Value.getMaxValue( nTypeY ));
		itrSrs = getIterator();
		while ( itrSrs.hasNext() ) {
			srs = itrSrs.next();
			mapVal = getMapVal( srs );
			mapKey = getKeys();
			itr = mapKey.keySet().iterator();
			while ( itr.hasNext() ) {
				Object key = itr.next();
				Object cur = mapVal.get( key );
				if ( cur == null ) {
					continue;
				}
				if ( dMin.compareTo( cur ) > 0 ) {
					dMin = new Value( cur );
				}
			}
		}
		setYRange( dMin.getObject(), dMax.getObject() );
	}

	/**
	* 保持しているデータから、X軸とY軸の範囲を設定します。
	*/
	public void setRange() {
		setXRange();
		setYRange();
	}

	/**
	* X軸の最大値を取得します。
	* @return	X軸の最大値
	*/
	public Object getXMax() {
		return xMax;
	}

	/**
	* X軸の最小値を取得します。
	* @return	X軸の最小値
	*/
	public Object getXMin() {
		return xMin;
	}

	/**
	* X軸の値を座標値に変換します。
	* @param	rect	描画する矩形
	* @param	x		格納されているキーオブジェクト
	* @param	y		格納されている値オブジェクト
	* @return	座標値
	*/
	public double getXPos( Rectangle rect, Object x, Object y ) {
		if ( x == null ) {
			return 0.0;
		}
		double dRet = new Value( x ).diff( xMin )
				/ new Value( xMax ).diff( xMin ) * rect.getWidth()
				+ rect.getX();
		return dRet;
	}

	/**
	* Y軸の最大値を取得します。
	* @return	Y軸の最大値
	*/
	public Object getYMax() {
		return yMax;
	}

	/**
	* Y軸の最小値を取得します。
	* @return	Y軸の最小値
	*/
	public Object getYMin() {
		return yMin;
	}

	/**
	* Y軸の値を座標値に変換します。
	* @param	rect	描画する矩形
	* @param	x		格納されているキーオブジェクト
	* @param	y		格納されている値オブジェクト
	* @return	座標値
	*/
	public double getYPos( Rectangle rect, Object x, Object y ) {
		if ( y == null ) {
			return 0.0;
		}
		return rect.getHeight() - new Value( y ).diff( yMin )
				/ new Value( yMax ).diff( yMin ) * rect.getHeight()
				+ rect.getY();
	}

	/**
	* 系列を追加します。
	* @param	srs		系列を表すオブジェクト
	*/
	public void addSeries( Object srs ) {
		TreeMap<Object,Value>	mapVal;
		mapVal = (TreeMap )get( srs );
		if ( mapVal == null ) {
			vecSrs.add( srs );
			mapVal = new TreeMap();
			put( srs, mapVal );
		}
	}

	/**
	* 保持しているデータを返します。
	* @return	保持データを格納しているMap
	*/
	public Map<Object,Number> getKeys() {
		return mapKey;
	}

	/**
	* 系列のデータを返します。
	* @param	srs		系列を表すオブジェクト
	*/
	public Map getMapVal( Object srs ) {
		return (TreeMap )get( srs );
	}

	/**
	* 系列の一覧を返します。
	* @return	系列の一覧
	*/
	public Iterator<Object> getIterator() {
		return vecSrs.iterator();
	}

	/**
	* データを追加します。
	* @param	srs		系列を表すオブジェクト
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( Object srs, Object x, Object y ) {
		addSeries( srs );
		TreeMap	mapVal = (TreeMap )get( srs );
		nTypeX = Value.getType( x );
		nTypeY = Value.getType( y );
		mapKey.put( x, Double.valueOf( 0.0 ));
		mapVal.put( x, y );
	}

	/**
	* データを追加します。
	* @param	srs		系列を表すオブジェクト
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( Object srs, double x, double y ) {
		add( srs, Double.valueOf( x ), Double.valueOf( y ));
	}

	/**
	* データを追加します。
	* @param	nSrs	系列を表すオブジェクト
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( int nSrs, Object x, double y ) {
		add( Integer.valueOf( nSrs ), x, Double.valueOf( y ));
	}

	/**
	* データを追加します。
	* @param	nSrs	系列を表すオブジェクト
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( int nSrs, double x, double y ) {
		add( nSrs, Double.valueOf( x ), y );
	}

	/**
	* データを追加します。
	* データがすでに存在する場合は、データを追加します。
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void stack( Object x, Object y ) {
		Number val = (Number )mapKey.get( x );
		if ( y != null ) {
			mapKey.put( x, Double.valueOf( ((Number )y).doubleValue() + val.doubleValue() ));
		}
	}

	// これ以降は、1グラフ1系列の場合の便利メソッドです。

	/**
	* データを格納します。
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( Object x, Object y ) {
		add( "", x, y );
	}

	/**
	* データを格納します。
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( Object x, double y ) {
		add( "", x, Double.valueOf( y ));
	}

	/**
	* データを格納します。
	* @param	x		X座標の値
	* @param	y		Y座標の値
	*/
	public void add( double x, double y ) {
		add( "", x, y );
	}

}

