package	jp.co.sds_corp.nanchart.graph;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
RaderGraphは、レーダーグラフを実装します。
*/
public class RaderGraph extends LineGraph
{
	/**
	* デフォルトの状態でレーダーグラフを構築します。
	*/
	public RaderGraph() {
		super();
	}

	/**
	* X軸とY軸の範囲を指定して、レーダーグラフを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public RaderGraph( double xMin, double xMax, double yMin, double yMax ) {
		super( xMin, xMax, yMin, yMax );
	}

	/**
	* X軸とY軸の範囲を指定して、レーダーグラフを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public RaderGraph( Object xMin, Object xMax, Object yMin, Object yMax ) {
		super( xMin, xMax, yMin, yMax );
	}

	/**
	* データ格納クラスを指定して、レーダーグラフを構築します。
	* @param	map		データ格納クラス
	*/
	public RaderGraph( GraphStorage map ) {
		super( map );
	}

	/**
	* 目盛り線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScaleLine( Graphics2D gr, Rectangle rect ) {
		for ( int i = 0; i < vecScale.size(); i++ ) {
			GraphScale scale = (GraphScale )vecScale.get( i );
			drawScaleLine( gr, rect, scale );
		}
	}

	/**
	* 目盛り線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	* @param	scale	目盛り
	*/
	public void drawScaleLine( Graphics2D gr, Rectangle rect, GraphScale scale ) {
		if ( center == null ) {
			init( rect );
		}
		if ( !scale.bLine ) {
			return;
		}
		Point2D	pts = null;
		Point2D	pte;
		Point2D	endP;
		Line2D	line = new Line2D.Double();

		gr.setColor( LineCol );
		gr.setStroke( scale.str );
		Number	key;
		String	msg;
		Iterator	itr = scale.keySet().iterator();
		while ( itr.hasNext() ) {
			key = (Number )itr.next();
			msg = (String )scale.get( key );
			int		nX;
			for ( nX = 0; nX < ((DefaultStorage)store).mapKey.size(); nX++ ) {
				pts = getPoint( rect, Integer.valueOf( nX-1 ), key );
				pte = getPoint( rect, Integer.valueOf( nX ), key );
				line.setLine( pts, pte );
				gr.draw( line );
			}
		}
	}

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScale( Graphics2D gr, Rectangle rect ) {
		for ( int i = 0; i < vecScale.size(); i++ ) {
			GraphScale scale = (GraphScale )vecScale.get( i );
			drawScale( gr, rect, scale );
		}
	}

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	* @param	scale	目盛り
	*/
	public void drawScale( Graphics2D gr, Rectangle rect, GraphScale scale ) {
		if ( center == null ) {
			init( rect );
		}

		Point2D	pts;
		Point2D	pte;
		Point2D	endP;
		Line2D	line = new Line2D.Double();

		Number	key;
		String	msg;
		boolean	bFirst = true;
		Iterator	itr = scale.keySet().iterator();
		while ( itr.hasNext() ) {
			key = (Number )itr.next();
			msg = (String )scale.get( key );
			for ( int nX = 0; nX < ((DefaultStorage)store).mapKey.size(); nX++ ) {
				drawTick( gr, rect, getPoint( rect, Integer.valueOf( nX ), key ) );
				if ( bFirst ) {
					endP = getPoint( rect, Integer.valueOf( nX ), store.getYMax() );
					gr.setColor( LineCol );
					gr.setStroke( scale.str );
					line.setLine( center, endP );
					gr.draw( line );
				}
			}
			bFirst = false;
			gr.setFont( scale.getFont() );
			pts = getPoint( rect, Integer.valueOf( 0 ), key );
			Rectangle2D recmsg = font.getStringBounds( msg, 0, msg.length(),
												gr.getFontRenderContext() );
			if ( scale.nPos == GraphScale.LEFT_BOTTOM ) {
				gr.drawString( msg, (int)(pts.getX()-recmsg.getWidth()-5),
								(int)(pts.getY()-recmsg.getCenterY()) );
			} else {
				gr.drawString( msg, (int)(pts.getX()+5),
								(int)(pts.getY()-recmsg.getCenterY()) );
			}
		}
	}

	/**
	* グラフをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	* @param	pt		出力するデータ
	*/
	public void drawTick( Graphics2D gr, Rectangle rect, Point2D pt ) {
		double	x1, y1, x2, y2;
		double	rt, sta;
		Line2D	line = new Line2D.Double();
		Arc2D	arc = new Arc2D.Double();
		double	rad = 5;
		Rectangle2D	recarc = new Rectangle2D.Double(
				pt.getX()-rad, pt.getY()-rad, rad*2, rad*2 );

		rt = (pt.getY() - center.getY()) / (pt.getX() - center.getX());
		sta = 0 - Math.toDegrees( Math.atan( rt ) );
		if ( Double.isNaN( sta )) {
		} else {
			arc.setArc( recarc, sta + 90, 180, Arc2D.OPEN );
			line.setLine( arc.getStartPoint(), arc.getEndPoint() );
			gr.draw( line );
		}
	}

	double	dSta = 90.0;
	double	dAng = 0.0;
	Point2D	center = null;
	Rectangle2D recall = null;

	/**
	* グラフを初期化します。
	* @param	rect	イメージ中のグラフの領域
	*/
	void init( Rectangle rect ) {
		if ( store.getXMax() == null ) {
			store.setXRange();
		}
		if ( store.getYMax() == null ) {
			store.setYRange();
		}
		center = new Point2D.Double( rect.getWidth()/2+rect.getX(), rect.getHeight()/2+rect.getY() );
		recall = new Rectangle2D.Double( 10+rect.getX(), 10+rect.getY(), rect.getWidth()-20, rect.getHeight()-20 );
		dAng = 360.0 / (double)((DefaultStorage)store).mapKey.size();
	}

	private Point2D getPoint( Rectangle rect, Object X, Object Y ) {
		Arc2D.Double	arc = new Arc2D.Double();
		double	x, y;
		double	rt;

		arc.setArc( recall, dSta - dAng * ((Number)X).doubleValue(), 0, Arc2D.OPEN );
		Rectangle2D recarc = arc.getBounds2D();
		Point2D endP = new Point2D.Double( recarc.getX(), recarc.getY() );

		if ( Y == null ) {
			return null;
		}
		rt = ((Number)Y).doubleValue() / ((Number)store.getYMax()).doubleValue();
		x = center.getX() + ( endP.getX() - center.getX() ) * rt;
		y = center.getY() + ( endP.getY() - center.getY() ) * rt;
		return new Point2D.Double( x, y );
	}

	/**
	* グラフ描画の後処理を行ないます。
	* @param	gr		描画するデバイス
	* @param	rect	描画する矩形
	* @param	srs		描画するグラフ
	* @param	curX	現在のX座標
	* @param	curY1	現在のY座標1
	* @param	curY2	現在のY座標2
	* @param	preX	前回のX座標
	* @param	preY1	前回のY座標1
	* @param	preY2	前回のY座標2
	*/
	public void endSeries( Graphics2D gr, Rectangle rect, Object srs,
				Object curX, Object curY1, Object curY2,
				Object preX, Object preY1, Object preY2 )
	{
		drawPoint( gr, rect, srs, curX, curY1, curY2, preX, preY1, preY2 );
	}

	/**
	* グラフ描画を行ないます。
	* @param	gr		描画するデバイス
	* @param	rect	描画する矩形
	* @param	srs		描画するグラフ
	* @param	curX	現在のX座標
	* @param	curY1	現在のY座標1
	* @param	curY2	現在のY座標2
	* @param	preX	前回のX座標
	* @param	preY1	前回のY座標1
	* @param	preY2	前回のY座標2
	*/
	public void drawPoint( Graphics2D gr, Rectangle rect, Object srs,
				Object curX, Object curY1, Object curY2,
				Object preX, Object preY1, Object preY2 )
	{
		Line2D	line = new Line2D.Double();
		int		cx, cy1, cy2;
		int		px, py1, py2;

		if ( center == null ) {
			init( rect );
		}

		Point2D endP = getPoint( rect, curX, store.getYMax() );
		Point2D curP = getPoint( rect, curX, curY2 );

		Color	lc = (Color)hashLineCol.get( srs );
		if ( lc == null ) {
			lc = LineCol;
		}
		if ( preX != null ) {
			gr.setColor( lc );
			gr.setStroke( str );
			Point2D preP = getPoint( rect, preX, preY2 );
			if ( preP != null && curP != null ) {
				line.setLine( preP, curP );
				gr.draw( line );
			}
		}
	}

}

