package	jp.co.sds_corp.nanchart.graph;

import jp.co.sds_corp.util.common.CmnDate;

/**
ローソク足グラフのデータを格納します。
<BR>
ローソク足データは、横軸に日付、縦軸には高値・安値・始値・終値のデータを
保持します。
@see	RosokuGraph
*/
public class RosokuData
{
	/** 日付 */
	CmnDate	dt;
	/** 高値 */
	double	high;
	/** 安値 */
	double	low;
	/** 始値 */
	double	start;
	/** 終値 */
	double	end;

	/**
	* ローソク足データを構築します。
	* @param	dt		日付
	* @param	high	高値
	* @param	low		安値
	* @param	start	始値
	* @param	end		終値
	*/
	public RosokuData( CmnDate dt, double high, double low,
						double start, double end ) {
		this.dt = dt;
		this.high = high;
		this.low = low;
		this.start = start;
		this.end = end;
	}

	/**
	* 設定されている日付を取得します。
	* @return	日付
	*/
	public CmnDate getDate() {
		return dt;
	}

	/**
	* 設定されている日付を取得します。
	* @return	高値
	*/
	public double getHigh() {
		return high;
	}

	/**
	* 設定されている日付を取得します。
	* @return	安値
	*/
	public double getLow() {
		return low;
	}

	/**
	* 設定されている日付を取得します。
	* @return	始値
	*/
	public double getStart() {
		return start;
	}

	/**
	* 設定されている日付を取得します。
	* @return	終値
	*/
	public double getEnd() {
		return end;
	}

	/**
	* 保持されている高値・安値・始値・終値の中で最大の値を取得します。
	* @return	最大値
	*/
	public double getMax() {
		double	dMax = Double.NEGATIVE_INFINITY;
		if ( high > dMax ) {
			dMax = high;
		}
		if ( low > dMax ) {
			dMax = low;
		}
		if ( start > dMax ) {
			dMax = start;
		}
		if ( end > dMax ) {
			dMax = end;
		}
		return dMax;
	}

	/**
	* 保持されている高値・安値・始値・終値の中で最小の値を取得します。
	* @return	最小値
	*/
	public double getMin() {
		double	dMin = Double.MAX_VALUE;
		if ( high < dMin ) {
			dMin = high;
		}
		if ( low < dMin ) {
			dMin = low;
		}
		if ( start < dMin ) {
			dMin = start;
		}
		if ( end < dMin ) {
			dMin = end;
		}
		return dMin;
	}
}

