package	jp.co.sds_corp.nanchart.graph;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
XScaleは、X軸の目盛りを実装します。
{@link #drawScale(Graphics2D,Rectangle)}によって、X軸の描画を実現します。
*/
public class XScale extends GraphScale
{
	private static final long serialVersionUID = 1L;

	/**
	* デフォルトの設定でXScaleScaleを構築します。
	*/
	public XScale() {
		super();
	}

	/**
	* 値の範囲を指定して、XScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public XScale( double min, double max ) {
		super( min, max );
	}

	/**
	* 値の範囲を指定して、XScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public XScale( Object min, Object max ) {
		super( min, max );
	}

	/**
	* 目盛り線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScaleLine( Graphics2D gr, Rectangle rect ) {
		if ( !bLine ) {
			return;
		}
		gr.setColor( getLineColor() );
		gr.setStroke( getLineStroke() );

		Object	key;
		String	msg;
		int		nPos;
		int		x, y;
		Iterator	itr = keySet().iterator();
		while ( itr.hasNext() ) {
			key = itr.next();
			msg = (String )get( key );

			nPos = (int )getPos( rect.getWidth(), key );
			x = (int )( rect.getX() + nPos );
			y = (int )( rect.getY() );
			gr.drawLine( x, y, x, (int )(y + rect.getHeight()) );
		}
	}

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScale( Graphics2D gr, Rectangle rect ) {
		gr.setColor( getColor() );
		gr.setStroke( getStroke() );

		if ( ang != 0 ) {
			AffineTransform	at = new AffineTransform();
			at.rotate( Math.toRadians( ang ) );
			font = font.deriveFont( at );
		}
		gr.setFont( font );

		Object	key;
		String	msg;
		int		nPos;
		int		x, y, x1, x2;
		if ( this.nPos == LEFT_BOTTOM ) {
			y = (int )( rect.getY() + rect.getHeight() );
		} else {
			y = (int )rect.getY();
		}
		if ( drawmax == null ) {
			x1 = (int )rect.getX();
			x2 = (int )( rect.getX() + rect.getWidth() );
		} else {
			x1 = (int )( rect.getX() + rect.getWidth() - getPos( rect.getWidth(), drawmin ));
			x2 = (int )( rect.getX() + rect.getWidth() - getPos( rect.getWidth(), drawmax ));
		}
		gr.drawLine( x1, y, x2, y );

		if ( this.nDir == DIR_OUTER ) {
			if ( this.nPos == LEFT_BOTTOM ) {
				y += ScaleLen;
			}
		} else {
			if ( this.nPos == RIGHT_TOP ) {
				y += ScaleLen;
			}
		}
		Iterator	itr = keySet().iterator();
		while ( itr.hasNext() ) {
			key = itr.next();
			msg = (String )get( key );

			Rectangle2D recmsg = font.getStringBounds( msg, 0, msg.length(),
												gr.getFontRenderContext() );
			nPos = (int )getPos( rect.getWidth(), key );

			x = (int )( nPos + rect.getX());
			int		rx, ry;
			if ( this.nPos == LEFT_BOTTOM ) {
				ry = y + (int )recmsg.getHeight() - 5;
			} else {
				ry = y - ScaleLen - 5;
			}
			if ( ang == 0 ) {
				gr.drawString( msg, x - (int )recmsg.getCenterX(), ry );
			} else {
				gr.drawString( msg, x, ry );
			}
			gr.drawLine( x, y, x, y - ScaleLen );
		}
	}

}
