package	jp.co.sds_corp.nanchart.graph;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
YScaleは、Y軸の目盛りを実装します。
{@link #drawScale(Graphics2D,Rectangle)}によって、Y軸の描画を実現します。
*/
public class YScale extends GraphScale
{
	private static final long serialVersionUID = 1L;

	/**
	* デフォルトの設定でYScaleScaleを構築します。
	*/
	public YScale() {
		super();
	}

	/**
	* 値の範囲を指定して、YScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public YScale( double min, double max ) {
		super( min, max );
	}

	/**
	* 値の範囲を指定して、YScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public YScale( Object min, Object max ) {
		super( min, max );
	}

	/**
	* 補助線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScaleLine( Graphics2D gr, Rectangle rect ) {
		if ( !bLine ) {
			return;
		}
		gr.setColor( getLineColor() );
		gr.setStroke( getLineStroke() );

		Double	key;
		String	msg;
		int		nPos;
		int		x, y;
		Iterator	itr = keySet().iterator();
		while ( itr.hasNext() ) {
			key = (Double )itr.next();
			msg = (String )get( key );

			nPos = (int )( rect.getHeight() - getPos( rect.getHeight(), key ) );
			x = (int )( rect.getX() );
			y = (int )( rect.getY() + nPos );
			gr.drawLine( x, y, (int )(x+rect.getWidth()), y );
		}
	}

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScale( Graphics2D gr, Rectangle rect ) {
		gr.setColor( getColor() );
		gr.setStroke( getStroke() );

		if ( ang != 0 ) {
			AffineTransform	at = new AffineTransform();
			at.rotate( Math.toRadians( ang ) );
			font = font.deriveFont( at );
		}
		gr.setFont( font );

		int		x, y, y1, y2;
		if ( this.nPos == LEFT_BOTTOM ) {
			x = (int )rect.getX();
		} else {
			x = (int )( rect.getX() + rect.getWidth() );
		}
		if ( drawmax == null ) {
			y1 = (int )rect.getY();
			y2 = (int )( rect.getY() + rect.getHeight() );
		} else {
			y1 = (int )( rect.getY() + rect.getHeight() - getPos( rect.getHeight(), drawmin ));
			y2 = (int )( rect.getY() + rect.getHeight() - getPos( rect.getHeight(), drawmax ));
		}

		gr.drawLine( x, y1, x, y2 );

		if ( this.nDir == DIR_OUTER ) {
			if ( this.nPos == LEFT_BOTTOM ) {
				x -= ScaleLen;
			} else {
				x += ScaleLen;
			}
		}
		Object	key;
		String	msg;
		int		nPos;
		Iterator	itr = keySet().iterator();
		while ( itr.hasNext() ) {
			key = itr.next();
			msg = (String )get( key );

			Rectangle2D recmsg = font.getStringBounds( msg, 0, msg.length(),
												gr.getFontRenderContext() );
			nPos = (int )( rect.getHeight() - getPos( rect.getHeight(), key ) );
			y = (int )(nPos + rect.getY());
			if ( this.nPos == LEFT_BOTTOM ) {
				gr.drawString( msg, (int )(x - recmsg.getWidth()),
							(int )( y - recmsg.getCenterY()) );
				gr.drawLine( x, y, x + ScaleLen, y );
			} else {
				gr.drawString( msg, x,
							(int )( y - recmsg.getCenterY()) );
				gr.drawLine( x - ScaleLen, y, x, y );
			}
		}
	}

}

